% function [D,Di,Dd]=genstereo(N,P,plt,spline,s);
%
% generates N stereo images of size 2*P (P for each eye) 
% and returns the result in N x 2*P matrix D
%
% D   =  data
% Di  =  indicator of true intensities
% Dd  =  indicator of true disparities
%
% if plt==1 it plots every disparity surface and image
% if spline==1 it uses a spline, otherwise one of 2 fixed disparities
% s = std dev of noise (default 0)

function [D,Di,Dd]=genstereo(N,P,plt,spl,s);

if nargin<5,
  s=0;
end;
D=zeros(N,2*P);
Dd=zeros(N,1);
Di=zeros(N,P);
for n=1:N,
  
  cpts=6;
  x=-1:0.6:2;
  y=rand(cpts,1)*0.3-0.15;
  xs=-.5:0.1:1.5;
  if spl,
    ys=spline(x,y,xs); 
  else
    qq=rand<0.5;
    rr=0.2*(qq-0.5);
    Dd(n)=qq;
    ys=ones(1,21)*rr;
  end;
    
  if plt,
    clg;
    subplot(211);
    plot(x,y,'o');
    hold on;
    plot(xs,ys,'r');
    plot([0 1],[-0.5/sqrt(3) 0.5/sqrt(3)],'--');
    plot([0 1],[0.5/sqrt(3) -0.5/sqrt(3)],'--');
    grid;
    axis('equal');
  end;
    
  % project onto lines with slopes 1/sqrt(3), -1/sqrt(3), i.e. 
  % 30 deg and -30 deg
  
  % take M random points xn with intensities 'in'
  M=8;
  xn=rand(M,1);
  in=rand(M,1);
  
  % and project onto surface 
  if spl,
    yn=spline(x,y,xn); 
  else
    yn=ones(M,1)*rr;
  end;
  
  pts=[xn yn];
  
  % form the two orthographic projections of this surface + canonical (0) projection
  
  proj1=[1; 1/sqrt(3)];
  proj2=[1; -1/sqrt(3)];
  p1=pts*proj1;
  p2=pts*proj2;
  p0=pts*[1; 0];
  
  % pixel coords

  pmin=-0.2;
  pmax=1.2;
  dpx=(pmax-pmin)/(P-1);
  px=pmin:dpx:pmax;
  sm=dpx/2;   % std deviation of blurring = 1/2 pixel
  
  I1=exp(-(px'*ones(1,M)-ones(P,1)*p1').^2/(2*sm^2))*in+randn(P,1)*s;
  I2=exp(-(px'*ones(1,M)-ones(P,1)*p2').^2/(2*sm^2))*in+randn(P,1)*s;
  I0=exp(-(px'*ones(1,M)-ones(P,1)*p0').^2/(2*sm^2))*in;
  
  if plt,
    subplot(413);
    show(I1');
    subplot(414);
    show(I2');
    drawnow;
  end;
  Di(n,:)=I0';
  D(n,:)=[I1' I2'];
  
end;

