/*

  SAMPREJ.C	Sample .MEX file corresponding to SAMPREJ.M

  function samp=samprej(mu,v);
  
  Sample from the positive part of a Gaussian truncated at 0 using 
  simple rejection sampling. This will *not* work well in the tails 
  of the Gaussian, that is, if mu/sqrt(v) << 0

*/

#include <math.h>
#include "mex.h"
#include "randn.c"

/* Input Arguments */

#define	MU_IN	prhs[0]
#define	V_IN	prhs[1]


/* Output Arguments */

#define	SAMP_OUT	plhs[0]


#define	max(A, B)	((A) > (B) ? (A) : (B))
#define	min(A, B)	((A) < (B) ? (A) : (B))


#define pi 3.14159265

static void samprej(
	double	*samp,
	double	*mu,
	double	*v
	)
{
  double sv;
  int done=0;

  sv=sqrt(v[0]);

  if (mu[0]/sv < -3) {
    printf("warning: samprej should not be used in Gaussian tails\n");
    return;
  }

  while (!done) {
    *samp=mu[0]+randn()*sv; 

    if (*samp>0)
      done=1;
  }
  
  return;

}

void mexFunction(
	int		nlhs,
	Matrix	*plhs[],
	int		nrhs,
	Matrix	*prhs[]
	)
{
	double	*samp, *mu, *v;
	unsigned int	m,n;

	/* Check for proper number of arguments */

	if (nrhs != 2) {
		mexErrMsgTxt("SAMPREJ requires two input arguments.");
	} else if (nlhs > 1) {
		mexErrMsgTxt("SAMPREJ requires one output argument.");
	}

	/* Check the dimensions of Y.  Y can be 4 X 1 or 1 X 4. */

	m = mxGetM(MU_IN);
	n = mxGetN(MU_IN);
	if (!mxIsNumeric(MU_IN) || mxIsComplex(MU_IN) || 
		!mxIsFull(MU_IN)  || !mxIsDouble(MU_IN) ||
		(max(m,n) != 1) || (min(m,n) != 1)) {
		mexErrMsgTxt("SAMPREJ requires that mu be a 1 x 1 vector.");
	}

	/* Create a matrix for the return argument */

	SAMP_OUT = mxCreateFull(1, 1, REAL);

	/* Assign pointers to the various parameters */

	samp = mxGetPr(SAMP_OUT);
	mu = mxGetPr(MU_IN);
	v = mxGetPr(V_IN);

	/* Do the actual computations in a subroutine */
	samprej(samp,mu,v);
	return;
}


