NL=1;         % nonlinearity of the unit(s) at the top layer: 1=rectified, 3=squashed 
              % In the simulation presented in the Phil Trans Roy Soc paper NL=3,
	      % However, the mex file for threegauss hasn't been implemented yet, 
	      % so to get a fast simulation you need to set NL=1.

K=6;          % size of bars grid (i.e. KxK pixels)
n1=1;         % number of units at top layer 
n2=24;        % numb of hidden units          
n3=K*K;       % numb of visible units

reps=16;      % Gibbs samples                  

eta=0.1;     % learning rate
lambda=0.01  % weight decay parameter
lambdav=0.001 % weight decay parameter for variances

eta2=eta;     % learning rate at top layer
eta3=eta;     % learning rate at bottom layer

numparam=n2*n1+n3*n2+n1+n2+n3+3; % total numb of params

savestep=100;   % save weights every this many steps
graphstep=200;  % plot progress graphics every this many steps (multiple of savestep)

init=1;       % flag to specify whether to re-initialize params or not XXXX 1
if init,      % initialize params and activities

  rgbn_init;
  
  %%% generate bars data
  N=1000;                       % number of data points
  [Data,Db,Dv]=genbars(N,K,1);         % generate this data
  cycles=10000;                 % number of data presentations
  params=zeros(fix(cycles/savestep),numparam); % record of all params during learning
  activ=zeros(N,n1+n2);
else
  initcyc=cycles;              
  cycles=2000;
  params=[params; zeros(fix(cycles/savestep),numparam)];  % append to old params
end;

tic;
for cyc=initcyc+1:initcyc+cycles,     % cycle over the data cycles times
  
  %% CLAMP ONE DATA VECTOR AND LEARN
  
  rgbn_core;

  if rem(cyc,20)==0,
    fprintf('.');
  end;
  
  if rem(cyc,savestep)==0,  % every 20th cycle print something and collect params
    fprintf('%g ',cyc);
    params(fix(cyc/savestep),:)=[W12(:)' W23(:)' b1(:)' b2(:)' b3(:)' v1 v2 v3];
  
    if rem(cyc,graphstep)==0,   % every graphstepth cycle show plot of what's going on
      clg;
      axes('position',[0.05,0.75,0.9,0.08]);
      hinton(W12');  
      title('Weights from top layer to middle layer');
      
      axes('position',[0.05,0.55,0.9,0.08]);
      hinton(b2');
      title('Biases at middle layer');

      axes('position',[0.5,0.5,0.01,0.01]);
      axis('off');
      th=text(0,0,'Weights from middle layer to bottom layer');
      set(th,'HorizontalAlignment','center');

      % plot weights from middle to bottom layer
      rng=ceil(max(abs(W23(:))));
      for k=1:n2,
	axes('position',[0.05+0.9*(k-1)/n2,0.35,0.9/n2,0.2]);
	hinton(reshape(W23(:,k),K,K),rng);
      end;
      
      % plot time evolution of weights from top to middle layer
      axes('position',[0.1,0.1,0.2,0.2])
      plot(params(1:fix(cyc/savestep),1:n1*n2));
      title('W12');
      
      % plot time evolution of *some* weights from middle to bottom layer
      st=fix(sqrt(n2*n3));
      wtsset=n1*n2+1:st:n1*n2+n2*n3;
      axes('position',[0.4,0.1,0.2,0.2])
      plot(params(1:fix(cyc/savestep),wtsset));
      title('W23');
      
      axes('position',[0.7,0.1,0.2,0.2])
      semilogy(params(1:fix(cyc/savestep),n1*n2+n2*n3+n1+n2+n3+(1:3)));
      title('log vars');
      
      fprintf(' time= %g\n',toc);
      drawnow;
    end;

  end;
end;
toc;